@@ -18,6 +18,30 @@ from colorama import Fore, Style, init
 from .config import config
 from .utils import get_connected_devices, get_wifi_ip
 
+
+def is_ci_environment() -> bool:
+    """
+    Check if running in a CI/CD environment.
+
+    Returns:
+        bool: True if running in CI, False otherwise
+    """
+    ci_indicators = [
+        'CI',  # Generic CI indicator
+        'GITHUB_ACTIONS',  # GitHub Actions
+        'GITLAB_CI',  # GitLab CI
+        'JENKINS_URL',  # Jenkins
+        'TRAVIS',  # Travis CI
+        'CIRCLECI',  # Circle CI
+        'BUILDKITE',  # Buildkite
+        'DRONE',  # Drone CI
+        'TF_BUILD',  # Azure DevOps
+        'BITBUCKET_BUILD_NUMBER',  # Bitbucket Pipelines
+    ]
+
+    return any(os.environ.get(var) for var in ci_indicators)
+
+
 # ── colorama ──────────────────────────────
 init(autoreset=True)
 green = Fore.GREEN
@@ -151,11 +175,17 @@ else:
 
 if platform != 'win':
     # ── terminal state capture ────────────────────────────────────────────────────
-    _ORIGINAL_STTY = subprocess.check_output(['stty', '-g']).decode().strip()
+    try:
+        _ORIGINAL_STTY = subprocess.check_output(['stty', '-g']).decode().strip()
+    except (subprocess.CalledProcessError, FileNotFoundError):
+        # No TTY available (CI environment, etc.) - disable terminal restoration
+        _ORIGINAL_STTY = None
+        logging.debug('No TTY available, terminal restoration disabled')
 
     def _restore_terminal() -> None:
         """Return terminal to the settings captured at startup."""
-        subprocess.run(['stty', _ORIGINAL_STTY], check=False)
+        if _ORIGINAL_STTY is not None:
+            subprocess.run(['stty', _ORIGINAL_STTY], check=False)
 
     def _sigint_handler(_sig, _frame) -> None:
         """Handle Ctrl-C: restore terminal, cleanup processes, then exit."""
@@ -273,6 +303,10 @@ def notify(title: str, message: str) -> None:
     Send a notification to the user's desktop.
     No support for Windows yet.
     """
+    # Check if notifications are enabled
+    if not config.SHOW_NOTIFICATIONS:
+        return
+
     try:
         if platform in {'linux', 'macosx'} and 'microsoft' not in platform_release:
             notification.notify(message=message, title=title)
@@ -445,7 +479,14 @@ def debug_and_livestream() -> None:
     """
     Executes `adb logcat` and `scrcpy` in parallel.
     Validates devices once at the start instead of in each subprocess.
+
+    In CI environments, this function gracefully exits without device validation.
     """
+    # Skip device operations in CI environments
+    if is_ci_environment():
+        logging.info('CI environment detected, skipping debug and livestream')
+        return
+
     # Early validation - exit immediately if no devices
     validate_devices_connected()
 
